import pwn
import os


class Binary:
	def __init__(self, name: str):
		self.name = name
		self.debug_name = name
		self.pwnfile = pwn.ELF(name, checksec=False)

	def set_executable(self) -> None:
		os.chmod(self.name, 0o777)
		if self.debug_name != self.name:
			os.chmod(self.debug_name, 0o777)



import pwn
import shutil
import os
import tempfile
import subprocess

import spwn.utils as utils
from spwn.binary import Binary
from spwn.libc import Libc
from spwn.loader import Loader
from spwn.configmanager import ConfigManager


class FileManager:
	def __init__(self, configs: ConfigManager):
		self.configs = configs
		self.binary = None
		self.libc = None
		self.loader = None
		self.other_binaries = []

	def auto_recognize(self, maybe_has_debug: bool) -> None:
		binaries = []
		libcs = []
		loaders = []
		other_libraries = []
		files = list(filter(lambda x: pwn.platform.architecture(x)[1] == "ELF", os.listdir()))
		self.other_binaries = files
		if not files:
			raise Exception("No ELFs found")

		for file in files:
			if file.startswith("libc"):
				libcs.append(file)
			elif file.startswith("lib"):
				other_libraries.append(file)
			elif file.startswith("ld"):
				loaders.append(file)
			else:
				binaries.append(file)

		# Binary
		if len(binaries) == 1:
			self.binary = Binary(binaries[0])
		elif len(binaries) > 1:
			self.binary = utils.ask_list_delete("Select binary", binaries, can_skip=False)
			self.binary = Binary(self.binary)
			other_libraries.extend(binaries)
		else:
			self.ask_all(files)
			return
		del files[files.index(self.binary.name)]

		# Libc
		if len(libcs) == 1:
			self.libc = Libc(libcs[0])
		else:
			libcs.extend(other_libraries)
			if not libcs:
				return

			self.libc = utils.ask_list_delete("Select libc", libcs, can_skip=True)
			if self.libc is None:
				return
			self.libc = Libc(self.libc)

			other_libraries = libcs
		del files[files.index(self.libc.name)]

		if maybe_has_debug:
			if os.path.exists(self.configs.debug_dir):
				if os.path.exists(os.path.join(self.configs.debug_dir, self.binary.name)):
					self.binary.debug_name = os.path.join(self.configs.debug_dir, self.binary.name)
				if os.path.exists(os.path.join(self.configs.debug_dir, "libc.so.6")):
					self.libc.debug_name = os.path.join(self.configs.debug_dir, self.libc.name)

		# Loader
		if len(loaders) == 1:
			self.loader = Loader(loaders[0])
		else:
			if not loaders:
				return
			self.loader = utils.ask_list_delete("Select loader", loaders, can_skip=True)
			if self.loader:
				self.loader = Loader(self.loader)
		del files[files.index(self.loader.name)]

	def ask_all(self, files: list[str]) -> None:
		if not files: raise Exception("No executables found")

		self.libc = None
		self.loader = None

		self.binary = utils.ask_list_delete("Select binary", files, can_skip=False)
		self.binary = Binary(self.binary)
		if files:
			self.libc = utils.ask_list_delete("Select libc", files, can_skip=True)
			if self.libc and files:
				self.libc = Libc(self.libc)
				self.loader = utils.ask_list_delete("Select loader", files, can_skip=True)
				if self.loader:
					self.loader = Loader(self.loader)
			else:
				self.loader = None
		else:
			self.libc = None
			self.loader = None

	def get_loader(self) -> None:
		ubuntu_version = self.libc.get_ubuntu_version_string()
		if ubuntu_version is None:
			print("[!] Cannot get ubuntu packet name for loader")
			return

		package_name = f"libc6_{ubuntu_version}_{self.libc.pwnfile.get_machine_arch()}.deb"
		package_url  = f"https://launchpad.net/ubuntu/+archive/primary/+files/{package_name}"
		tempdir = tempfile.mkdtemp()

		print("[+] Downloading loader")
		if not utils.download_package(package_url, tempdir):
			shutil.rmtree(tempdir)
			return

		print("[+] Extracting loader")
		if not utils.extract_deb(tempdir):
			shutil.rmtree(tempdir)
			return

		if not utils.find_and_extract_data(tempdir):
			shutil.rmtree(tempdir)
			return

		loader_path = utils.find_loader(tempdir)
		if loader_path is None:
			shutil.rmtree(tempdir)
			return

		new_loader_path = os.path.join(self.configs.debug_dir, "ld-linux.so.2")
		shutil.copyfile(loader_path, new_loader_path)
		self.loader = Loader(new_loader_path)

		shutil.rmtree(tempdir)
		return

	def patchelf(self) -> None:
		if self.loader:
			try:
				subprocess.check_call(["patchelf", "--set-interpreter", f"./{self.loader.debug_name}", "--set-rpath", f"./{self.configs.debug_dir}", self.binary.debug_name])
			except subprocess.CalledProcessError:
				print("[!] patchelf failed")
		else:
			try:
				subprocess.check_call(["patchelf", "--set-rpath", f"./{self.configs.debug_dir}", self.binary.debug_name])
			except subprocess.CalledProcessError:
				print("[!] patchelf failed")



import pwn
import requests
import os
import subprocess


def ask_list_delete(msg: str, options: list[str], can_skip: bool) -> str | None:
	if len(options) == 1 and not can_skip:
		ans = options[0]
		del options[0]
		return ans

	print(f"{msg}:")
	for n, op in enumerate(options):
		print(f"[{n + 1}] {op}")

	if can_skip:
		print("( Enter to skip )")

	while True:
		resp = input("> ").strip()
		if not resp and can_skip:
			return None

		if resp.isdigit():
			resp = int(resp) - 1
			if 0 <= resp < len(options):
				break

		print("[!] Invalid option")

	ans = options[resp]
	del options[resp]

	return ans


def ask_list(msg: str, options: list[str], can_skip: bool) -> str | None:
	if len(options) == 1:
		ans = options[0]
		return ans

	print(f"{msg}:")
	for n, op in enumerate(options):
		print(f"[{n + 1}] {op}")

	while True:
		resp = input("> ").strip()
		if not resp and can_skip:
			return None

		if resp.isdigit():
			resp = int(resp) - 1
			if 0 <= resp < len(options):
				break

		print("[!] Invalid option")

	ans = options[resp]
	del options[resp]

	return ans


def ask_string(msg: str, can_skip: bool) -> str | None:
	resp = input(f"{msg} > ")
	if resp: return resp
	elif can_skip: return None

	while True:
		print("[!] Canno be empty")
		resp = input("> ")
		if resp:
			return resp


def download_package(package_url: str, tempdir: str) -> bool:
	try:
		r = requests.get(package_url)
	except Exception:
		print(f"[!] Cannot get package from {package_url}")
		return False
	if r.status_code != 200:
		print(f"[!] Cannot get package from {package_url} ( ERROR: {r.status_code} )")
		return False

	with open(os.path.join(tempdir, "libc.deb"), "wb") as f:
		f.write(r.content)

	return True


def extract_deb(tempdir: str) -> bool:
	try:
		subprocess.check_call(["ar", "x", os.path.join(tempdir, "libc.deb"), "--output=" + tempdir])
		return True
	except subprocess.CalledProcessError:
		print("[!] Cannot extract .deb package")
		return False


def find_and_extract_data(tempdir: str) -> bool:
	to_extract = None
	for file in os.listdir(tempdir):
		if file.startswith("data."):
			to_extract = file
			break

	if to_extract is None:
		print("[!] Cannot find data archive")
		return False

	try:
		subprocess.check_call(["tar", "xf", os.path.join(tempdir, to_extract), "-C", tempdir])
	except subprocess.CalledProcessError:
		print("[!] Cannot extract data archive")
		return False

	return True


def find_loader(tempdir: str) -> str | None:
	for parent, dirs, files in os.walk(tempdir):
		for file in files:
			if not file.startswith("ld-linux"): continue
			file = os.path.join(parent, file)
			if os.path.isfile(file) and pwn.platform.architecture(file)[1] == "ELF":
				return file

	print("[!] Cannot find loader inside deb package")
	return None



import subprocess
import importlib
import os

from spwn.configmanager import ConfigManager
from spwn.filemanager import FileManager


class CustomAnalyzer:
	def __init__(self, configs: ConfigManager, files: FileManager):
		self.configs = configs
		self.files = files

	def pre_analysis(self) -> None:
		for command, timeout in self.configs.preanalysis_commands:
			self.run_command(command, timeout)
		for script in self.configs.preanalysis_scripts:
			self.run_script(script)

	def post_analysis(self) -> None:
		for command, timeout in self.configs.postanalysis_commands:
			self.run_command(command, timeout)
		for script in self.configs.postanalysis_scripts:
			self.run_script(script)

	def run_command(self, command: str, timeout: int | bool | None) -> None:
		command = command.format(binary=self.files.binary.name, debug_binary=self.files.binary.debug_name)
		if timeout is not False:
			print(f"[*] {command}")
			if timeout:
				try:
					# Use `exec command`, otherwise, because of `shell=True`, the process won't get killed on timeout
					p = subprocess.run(f"exec {command}", shell=True, timeout=timeout, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, encoding="latin-1")
					print(p.stdout)
				except subprocess.TimeoutExpired:
					print("[!] Timeout")
			else:
				p = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, encoding="latin-1")
				print(p.stdout)
		else:
			subprocess.Popen(command, shell=True, start_new_session=True)

	def run_script(self, script: str):
		if script.startswith("~"):
			script = os.path.expanduser(script)
		elif not script.startswith("/"):
			script = os.path.join(self.configs.config_path, script)

		if not os.path.isfile(script):
			raise FileNotFoundError(f"Cannot find custom script {script}")

		spec = importlib.util.spec_from_file_location("main", script)
		module = importlib.util.module_from_spec(spec)
		spec.loader.exec_module(module)
		module.main(self.files)



import os
import requests
import json


default_configs = {
	"debug_dir": "debug_dir",
	"script_file": "a.py",
	"pwn_process": "r",
	"tab": "\t",
	"template_file": "~/.config/spwn/template.py",
	"custom_template_prefix": "template_",
	"suppress_warnings": False,
	"yara_rules": "~/.config/spwn/findcrypt3.rules",
	"preanalysis_commands": [],
	"postanalysis_commands": [],
	"preanalysis_scripts": [],
	"postanalysis_scripts": [],
	"idafree_command": "",
	"decompiler_command": ""
}

default_template = '''
from pwn import *

binary_name = "{binary}"
exe  = ELF(binary_name, checksec=True)
libc = ELF("{libc}", checksec=False)
context.binary = exe

ru  = lambda *x, **y: r.recvuntil(*x, **y)
rl  = lambda *x, **y: r.recvline(*x, **y)
rc  = lambda *x, **y: r.recv(*x, **y)
sla = lambda *x, **y: r.sendlineafter(*x, **y)
sa  = lambda *x, **y: r.sendafter(*x, **y)
sl  = lambda *x, **y: r.sendline(*x, **y)
sn  = lambda *x, **y: r.send(*x, **y)

if args.REMOTE:
	r = connect("")
elif args.GDB:
	r = gdb.debug(f"{debug_dir}/{{binary_name}}", """
		c
	""", aslr=False)
else:
	r = process(f"{debug_dir}/{{binary_name}}")

{interactions}


r.interactive()
'''[1:-1]


class ConfigGenerator:
	def maybe_create_config(self):
		self.create_config_files()
		self.download_yara_rules()

	def create_config_files(self):
		config_dir = os.path.expanduser("~/.config")
		if not os.path.exists(config_dir):
			os.mkdir(config_dir)

		configs_dir = os.path.expanduser("~/.config/spwn")
		if not os.path.exists(configs_dir):
			os.mkdir(configs_dir)

		configs_file = os.path.expanduser("~/.config/spwn/config.json")
		template_file = os.path.expanduser("~/.config/spwn/template.py")

		if not os.path.exists(configs_file):
			with open(configs_file, "w") as f:
				json.dump(default_configs, f, indent='\t')
		else:
			with open(configs_file) as f:
				user_configs = json.load(f)
			new_configs = default_configs | user_configs
			with open(configs_file, "w") as f:
				json.dump(new_configs, f, indent='\t')

		if not os.path.exists(template_file):
			with open(template_file, "w") as f:
				f.write(default_template)

	def download_yara_rules(self):
		rules_path = os.path.expanduser("~/.config/spwn/findcrypt3.rules")
		if not os.path.exists(rules_path):
			r = requests.get("https://raw.githubusercontent.com/polymorf/findcrypt-yara/master/findcrypt3.rules")
			assert r.status_code == 200, "Cannot download yara rules"

			with open(rules_path, "w") as f:
				f.write(r.text)



import json
import os
from typing import Any


class ConfigManager:
	def __init__(self, config_path: str, template: str | None):
		config_path = os.path.expanduser(config_path)
		configs = json.load(open(config_path))
		for conf in configs:
			if isinstance(configs[conf], str) and configs[conf].startswith("~/"):
				configs[conf] = os.path.expanduser(configs[conf])

		configs["config_path"] = config_path

		if template is not None:
			template_path = os.path.dirname(configs["template_file"])
			custom_template = os.path.join(template_path, configs["custom_template_prefix"] + template + ".py")
			if not os.path.exists(custom_template):
				print(f"[ERROR] Template file not found ({custom_template})")
				exit(1)

			configs["template_file"] = custom_template

		super().__setattr__("configs", configs)

	def __getattribute__(self, key: str) -> None:
		configs = super(ConfigManager, self).__getattribute__("configs")
		if key in configs:
			return configs[key]
		else:
			raise KeyError

	def __setattr__(self, key: str, value: Any) -> None:
		configs = super(ConfigManager, self).__getattribute__("configs")
		if key in configs:
			configs[key] = value
		else:
			raise KeyError



import argparse
import os
import shutil
import sys

sys.argv = ["NOTERM"] + sys.argv  # HACK to prevent pwntools to messup the terminal, the added arg will be removed by pwnlib

from spwn.filemanager import FileManager
from spwn.analyzer import Analyzer
from spwn.scripter import Scripter
from spwn.configmanager import ConfigManager
from spwn.customanalyzer import CustomAnalyzer
from spwn.configgenerator import ConfigGenerator


CONFIG_PATH = os.path.expanduser("~/.config/spwn/config.json")


class Spwn:
	def __init__(self, create_interactions: bool, no_decompiler: bool, script_only: bool, interactions_only: bool):
		if not interactions_only:
			self.create_interactions = create_interactions or script_only
			self.no_decompiler = no_decompiler
			self.script_only = script_only
			self.check_dependencies()
			self.files = FileManager(configs)
			self.files.auto_recognize(self.script_only)

			print("[*] Binary:", self.files.binary.name)
			if self.files.libc: print("[*] Libc:  ", self.files.libc.name)
			else: print("[!] No libc")
			if self.files.loader: print("[*] Loader:", self.files.loader.name)
			else: print("[!] No loader")
			if self.files.other_binaries: print("[*] Other: ", self.files.other_binaries)
			print()
		else:
			self.files = None

		self.run()

	def run(self) -> None:
		if self.files:
			if not self.script_only:
				analyzer = Analyzer(configs, self.files)
				custom_analyzer = CustomAnalyzer(configs, self.files)
				analyzer.pre_analysis(open_decompiler=not self.no_decompiler)
				custom_analyzer.pre_analysis()
				print()

				if self.files.libc:
					self.create_debug_dir()
					self.populate_debug_dir()
					self.files.libc.maybe_unstrip()

					if self.files.loader is None:
						self.files.get_loader()
					if self.files.loader is not None:
						self.files.loader.set_executable()

					self.files.patchelf()

				self.files.binary.set_executable()
				analyzer.post_analysis()
				custom_analyzer.post_analysis()

			self.scripter = Scripter(configs, self.files, create_interactions=self.create_interactions)
			self.scripter.create_script()
			self.create_script_file()
			self.scripter.save_script()
		else:
			self.scripter = Scripter(configs)
			self.scripter.create_menu_interaction_functions()
			self.scripter.dump_interactions()

	def check_dependencies(self) -> None:
		deps = ["patchelf", "file"]
		semi_deps = {
			"eu-unstrip": "elfutils",
			"seccomp-tools": "seccomp-tools",
			"cwe_checker": "cwe_checker (https://github.com/fkie-cad/cwe_checker)"
		}

		err = False
		for dep in deps:
			if not shutil.which(dep):
				print(f"[ERROR] Please install {dep}")
				err = True

		if not configs.suppress_warnings:
			for dep in semi_deps:
				if not shutil.which(dep):
					print(f"[WARNING] Please install {semi_deps[dep]}")

		if err:
			exit(1)

	def create_debug_dir(self) -> None:
		global configs
		while os.path.exists(configs.debug_dir):
			if os.path.isdir(configs.debug_dir):
				print(f'[!] {configs.debug_dir} directory already exists, enter a new name or press enter to use it anyways')
				new_dir = input("> ").strip()
				if not new_dir:
					break
				configs.debug_dir = new_dir
			else:
				print(f'[!] {configs.debug_dir} file already exists, enter a new name')
				new_dir = input("> ").strip()
				if not new_dir:
					print(f'[!] {configs.debug_dir} is not a directory and cannot be overwritten')
				else:
					configs.debug_dir = new_dir

		if not os.path.exists(configs.debug_dir):
			os.mkdir(configs.debug_dir)

	def create_script_file(self) -> None:
		global configs
		while os.path.exists(configs.script_file):
			if os.path.isfile(configs.script_file):
				print(f'[!] {configs.script_file} file already exists, enter a new name or press enter to overwrite')
				new_file = input("> ").strip()
				if not new_file:
					break
				configs.script_file = new_file
			else:
				print(f'[!] {configs.script_file} already exists, enter a new name')
				new_file = input("> ").strip()
				if not new_file:
					print(f'[!] {configs.script_file} is not a file and cannot be overwritten')
				else:
					configs.script_file = new_file

	def populate_debug_dir(self) -> None:
		shutil.copy(self.files.binary.name, os.path.join(configs.debug_dir, self.files.binary.name))
		self.files.binary.debug_name = os.path.join(configs.debug_dir, self.files.binary.name)

		shutil.copy(self.files.libc.name, os.path.join(configs.debug_dir, "libc.so.6"))
		self.files.libc.debug_name = os.path.join(configs.debug_dir, "libc.so.6")

		if self.files.loader:
			shutil.copy(self.files.loader.name, os.path.join(configs.debug_dir, "ld-linux.so.2"))
			self.files.loader.debug_name = os.path.join(configs.debug_dir, "ld-linux.so.2")

		for binary in self.files.other_binaries:
			shutil.copy(binary, os.path.join(configs.debug_dir, binary))


def main():
	parser = argparse.ArgumentParser(
		prog="spwn",
		description="spwn is a tool to quickly start a pwn challenge, for more informations check https://github.com/MarcoMeinardi/spwn"
	)

	parser.add_argument(
		"-i", "--inter",
		action="store_true",
		default=False,
		help="Interactively create interaction functions"
	)

	parser.add_argument(
		"-so", "--sonly",
		action="store_true",
		default=False,
		help="Create the interaction script without analyzing the binary"
	)

	parser.add_argument(
		"-io", "--ionly",
		action="store_true",
		default=False,
		help="Create the interaction functions, without doing any analysis"
	)

	parser.add_argument(
		"-nd", "--nodecomp",
		action="store_true",
		default=False,
		help="Don't open the decompiler"
	)

	parser.add_argument(
		"--config",
		action="store_true",
		default=False,
		help="Setup configs and quit"
	)

	parser.add_argument(
		"others",
		nargs=argparse.REMAINDER,
		help="You can avoid typing the hyphens and/or specify the template"
	)

	args = parser.parse_args(sys.argv[1:])
	others = set(args.others)

	possible_arguments = {
		"ionly": "ionly",
		"so": "sonly",
		"sonly": "sonly",
		"io": "ionly",
		"interactions": "inter",
		"inter": "inter",
		"i": "inter",
		"nodecomp": "nodecomp",
		"nd": "nodecomp"
	}

	for arg in possible_arguments:
		if arg in others:
			setattr(args, possible_arguments[arg], True)
			others.remove(arg)

	ConfigGenerator().maybe_create_config()
	if args.config or "config" in others:
		print("[*] Setup completed")
	else:
		global configs
		template = others.pop() if others else None
		configs = ConfigManager(CONFIG_PATH, template)

		Spwn(create_interactions=args.inter, no_decompiler=args.nodecomp, script_only=args.sonly, interactions_only=args.ionly)



from spwn.filemanager import FileManager
from spwn.configmanager import ConfigManager
import spwn.utils as utils


class Scripter:
	def __init__(self, configs: ConfigManager, files: FileManager | None = None, create_interactions: bool = False):
		self.interactions = ""
		self.configs = configs
		if files:
			self.files = files
			self.create_interactions = create_interactions
			with open(self.configs.template_file) as f:
				self.template = f.read()

	def create_script(self) -> None:
		if self.create_interactions:
			if "{interactions}" not in self.template:
				print("[!] Template does not contain {interactions} placeholder")
			else:
				self.create_menu_interaction_functions()

		if self.files.libc:
			self.script = self.template.format(
				binary=self.files.binary.name,
				libc=self.files.libc.name,
				debug_dir=self.configs.debug_dir,
				interactions=self.interactions
			)

		else:
			# Remove libc line from template
			self.template = "\n".join(filter(lambda x: "{libc}" not in x, self.template.splitlines()))

			self.script = self.template.format(
				binary=self.files.binary.name,
				debug_dir=".",
				interactions=self.interactions
			)

	def save_script(self) -> None:
		with open(self.configs.script_file, "w") as f:
			f.write(self.script)

	def dump_interactions(self):
		print()
		print("-" * 50)
		print(self.interactions, end='')
		print("-" * 50)

	def create_menu_interaction_functions(self) -> None:
		menu_recvuntil = input("Menu recvuntil > ")
		if not menu_recvuntil: return
		while True:
			fun_name = self.ask_function_name()
			if fun_name is None: break
			function = InteractionFunction(self.configs, menu_recvuntil, fun_name)
			function.build()
			self.interactions += function.emit_function() + "\n"

	def ask_function_name(self) -> str | None:
		name = input("Function name ( Empty to end ) > ")
		return name if name else None


class InteractionFunction:
	def __init__(self, configs: ConfigManager, menu_recvuntil: str, name: str):
		self.configs = configs
		self.menu_recvuntil = menu_recvuntil
		self.name = name
		self.variables = []

	def build(self) -> None:
		self.ask_menu_option()
		while self.ask_variable():
			...

	def ask_menu_option(self) -> None:
		while True:
			self.menu_option = input("Menu option > ")
			if self.menu_option: break
			print("[!] Cannot be empty")

	def ask_variable(self) -> bool:
		variable_name = utils.ask_string("Variable name ( Empty to end function )", can_skip=True)
		if not variable_name: return False
		variable_type = utils.ask_list("Variable type", ["int", "bytes"], can_skip=False)
		variable_interaction = utils.ask_string("Send after", can_skip=False)

		if variable_type == "int":
			self.variables.append(IntVariable(self.configs, variable_name, variable_interaction))
		elif variable_type == "bytes":
			self.variables.append(ByteVariable(self.configs, variable_name, variable_interaction))
		else:
			assert False, "???"

		return True

	def emit_function(self) -> str:
		function = f'def {self.name}({", ".join(v.name for v in self.variables)}):\n'
		function += f'{self.configs.tab}{self.configs.pwn_process}.sendlineafter(b"{self.menu_recvuntil}", b"{self.menu_option}")\n'
		for v in self.variables:
			function += f"{self.configs.tab}{v.emit_interaction()}\n"

		return function


class AbstractVariable:
	def __init__(self, configs: ConfigManager, name: str, interaction: str):
		self.configs = configs
		self.name = name
		self.interaction = interaction


class IntVariable(AbstractVariable):
	def __init__(self, configs: ConfigManager, name: str, interaction: str):
		super().__init__(configs, name, interaction)

	def emit_interaction(self) -> str:
		if self.interaction:
			return f'{self.configs.pwn_process}.sendlineafter(b"{self.interaction}", b"%d" % {self.name})'
		else:
			return f'{self.configs.pwn_process}.sendline(b"%d" % {self.name})'


class ByteVariable(AbstractVariable):
	def __init__(self, configs: ConfigManager, name: str, interaction: str):
		super().__init__(configs, name, interaction)

	def emit_interaction(self) -> str:
		if self.interaction:
			return f'{self.configs.pwn_process}.sendlineafter(b"{self.interaction}", {self.name})'
		else:
			return f'{self.configs.pwn_process}.sendline({self.name})'



import pwn
import re
import shutil

from spwn.binary import Binary


class Libc(Binary):
	def __init__(self, name: str):
		super().__init__(name)
		self.version = self.get_libc_version()

	def get_libc_version(self) -> str:
		with open(self.name, "rb") as f:
			content = f.read().decode("latin-1")
		match = re.search(r"stable release version (\d\.\d+)", content)
		if match:
			return match.group(1)

		return pwn.platform.libc_ver(self.name)[1]

	def has_debug_info(self) -> bool:
		return bool(self.pwnfile.get_section_by_name(".debug_info"))

	def maybe_unstrip(self) -> None:
		if self.has_debug_info(): return

		if not shutil.which("eu-unstrip"):
			print("[ERROR] eu-unstrip not found, please install elfutils")
			return

		print("[+] Trying to unstrip libc")
		pwn.context.log_level = "warning"
		if pwn.libcdb.unstrip_libc(self.debug_name):
			self.unstripped = True
			print("[*] Libc unstripped")
		else:
			self.unstripped = False
			print("[!] Failed to unstrip libc")
		pwn.context.log_level = "info"

	def get_ubuntu_version_string(self) -> str | None:
		with open(self.name, "rb") as f:
			content = f.read()
		match = re.search(r"\d+\.\d+-\d+ubuntu\d+(\.\d+)?".encode(), content)

		if match:
			return match.group().decode()

		if not self.unstripped:
			return None

		with open(self.debug_name, "rb") as f:
			content = f.read()
		match = re.search(r"\d+\.\d+-\d+ubuntu\d+(\.\d+)?".encode(), content)

		if match:
			return match.group().decode()
		else:
			return None



import setuptools


with open("README.md") as f:
	long_description = f.read()

setuptools.setup(
	name="spwn",
	version="1.2.5",
	author="Chino",
	description="Automatic tool to quickly start a pwn CTF challenge",
	long_description=long_description,
	long_description_content_type="text/markdown",
	classifiers=[
		"Environment :: Console",
		"Operating System :: POSIX :: Linux",
		"Programming Language :: Python :: 3",
		"Topic :: Security",
		"Topic :: Software Development :: Code Generators"
	],
	packages=["spwn"],
	install_requires=[
		"pwntools",
		"yara-python"
	],
	entry_points={
		"console_scripts": ["spwn = spwn.spwn:main"]
	},

	url="https://github.com/MarcoMeinardi/spwn"
)



import subprocess
import shutil
import re
import yara

from spwn.filemanager import FileManager
from spwn.configmanager import ConfigManager

dangerous_functions_check = ["system", "execve", "gets", "ptrace", "memfrob", "strfry"]


class Analyzer:
	def __init__(self, configs: ConfigManager, files: FileManager):
		self.configs = configs
		self.files = files

	def pre_analysis(self, open_decompiler: bool) -> None:
		self.run_file()
		self.run_checksec()
		self.print_libc_version()
		self.print_dangerous_functions()
		self.run_yara()
		if open_decompiler:
			self.open_decompiler()
		self.run_cwe_checker()

	def post_analysis(self) -> None:
		self.check_and_print_seccomp()

	def run_file(self) -> None:
		print(f"[*] file {self.files.binary.name}")
		file_output = subprocess.check_output(["file", self.files.binary.name], encoding="latin1")
		type_and_arch = re.search(r"(ELF.*?), (.*?),", file_output)
		linking = re.search(r"(dynamically linked|statically linked)", file_output)
		debug_info = re.search(r"with debug_info", file_output)
		stripped = re.search(r"(not )?stripped", file_output)

		if not type_and_arch or not linking or not stripped:
			print(file_output.strip())
		else:
			print(type_and_arch.group(1))
			print(type_and_arch.group(2))
			print(linking.group())
			if debug_info:
				print(debug_info.group())
			print(stripped.group())

	def run_checksec(self) -> None:
		print(f"[*] checksec {self.files.binary.name}")
		print(self.files.binary.pwnfile.checksec(banner=False))

	def print_libc_version(self) -> None:
		if self.files.libc:
			print(f"Libc version: {self.files.libc.version}")

	def print_dangerous_functions(self) -> None:
		dangerous_functions = list(filter(lambda x: x in dangerous_functions_check, self.files.binary.pwnfile.sym))

		if dangerous_functions:
			print("[!] There are some dangerous functions:")
			print(" ".join(dangerous_functions))

	def run_yara(self) -> None:
		rules = yara.compile(self.configs.yara_rules)
		matches = rules.match(self.files.binary.name)

		if matches:
			print("[!] yara found something")
			for match in matches:
				addresses = [instance.offset for string_match in match.strings for instance in string_match.instances]
				print(f'[*] {match.rule} at {", ".join(map(hex, addresses))}')
			print()

	def open_decompiler(self) -> None:
		if self.configs.idafree_command and self.files.binary.pwnfile.arch in ["amd64", "i386"]:
			subprocess.Popen(self.configs.idafree_command.format(binary=self.files.binary.name), shell=True, start_new_session=True)
		elif self.configs.decompiler_command:
			subprocess.Popen(self.configs.decompiler_command.format(binary=self.files.binary.name), shell=True, start_new_session=True)

	def check_and_print_seccomp(self) -> None:
		for function in self.files.binary.pwnfile.sym:
			if "seccomp" in function or "prctl" in function:
				self.run_seccomptools()
				break

	def run_seccomptools(self) -> None:
		print("[!] Possible seccomp found")
		if not shutil.which("seccomp-tools"):
			if not self.configs.suppress_warnings:
				print("[ERROR] seccomp-tools not found, either it is not installed or is not in your $PATH")
		else:
			print(f"[*] seccomp-tools dump ./{self.files.binary.debug_name} < /dev/null")
			try:
				print(subprocess.check_output(["seccomp-tools", "dump", f"./{self.files.binary.debug_name}"], timeout=1, stdin=subprocess.DEVNULL, stderr=subprocess.STDOUT, encoding="latin1"))
			except subprocess.TimeoutExpired as e:
				print(f"[!] {e}")

	def run_cwe_checker(self) -> None:
		if not shutil.which("cwe_checker"):
			if not self.configs.suppress_warnings:
				print("[ERROR] cwe_checker not found, either it is not installed or is not in your $PATH")
		else:
			print(f"[*] cwe_checker {self.files.binary.name} (press Ctrl+C to stop)")
			try:
				print(subprocess.check_output(["cwe_checker", self.files.binary.name], encoding="latin1"))
			except KeyboardInterrupt:
				pass
